@@ -1,49 +1,32 @@
 import lxml.etree as ET
 import os
 
-from pycobertura.utils import extrapolate_coverage, reconcile_lines
+from pycobertura.utils import (
+    extrapolate_coverage,
+    reconcile_lines,
+    hunkify_lines,
+)
 
 
 class Cobertura(object):
     """
     An XML cobertura parser.
     """
-    def __init__(self, xml_source, base_path=None):
+    def __init__(self, xml_path, base_path=None):
         """
-        Initialize a Cobertura report given an XML file `xml_source` that is
-        in the cobertura format.
+        Initialize a Cobertura report given a path to an XML file `xml_path`
+        that is in the cobertura format.
 
         The optional argument `base_path` can be provided to resolve the path
-        to the source code. `base_path` will prefix the filename attribute
-        found in the Cobertura report.
-
-        `xml_source` may be:
-        - a path to an XML file
-        - an open file object
-        - an XML string
+        to the source code. If omitted, `base_path` will be set to
+        `os.path.dirname(xml_source)`.
         """
+        if base_path is None:
+            base_path = os.path.dirname(xml_path)
+
         self.base_path = base_path
-        self.source = xml_source
-        self.xml = self._parse(xml_source)
-
-    def _parse(self, xml_source):
-        if hasattr(xml_source, 'startswith'):
-            if xml_source.startswith('<?xml'):
-                return self._parse_xml_string(xml_source)
-
-            return self._parse_xml_file(xml_source)
-
-        elif hasattr(xml_source, 'read'):
-            return self._parse_xml_fileobj(xml_source)
-
-    def _parse_xml_file(self, xml_file):
-        return ET.parse(xml_file).getroot()
-
-    def _parse_xml_fileobj(self, xml_fileobj):
-        return self._parse_xml_string(xml_fileobj.read())
-
-    def _parse_xml_string(self, xml_string):
-        return ET.fromstring(xml_string)
+        self.xml_path = xml_path
+        self.xml = ET.parse(xml_path).getroot()
 
     def _get_element_by_class_name(self, class_name):
         return self.xml.xpath("//class[@name='%s'][1]" % class_name)[0]
@@ -135,7 +118,7 @@ class Cobertura(object):
         `source`: actual source code for line number `lineno`
         `status`: True (hit) or False (miss)
         """
-        filename = self.filename(class_name)
+        filename = self.filepath(class_name)
 
         if not os.path.exists(filename):
             return [(0, '%s not found' % filename, None)]
@@ -198,18 +181,22 @@ class Cobertura(object):
 
     def filename(self, class_name):
         """
-        Return the filename of the class `class_name`. If `base_path` was
-        provided in the constructor, it will be prefixed to the filename using
-        `os.path.join`.
+        Return the filename of the class `class_name` as found in the Cobertura
+        report.
         """
         el = self._get_element_by_class_name(class_name)
         filename = el.attrib['filename']
+        return filename
 
-        if self.base_path is None:
-            return filename
-
-        path = os.path.join(self.base_path, filename)
-        return path
+    def filepath(self, class_name):
+        """
+        Return the filesystem path to the actual class file. It uses the
+        `base_path` value initialized in the constructor by prefixing it to the
+        class filename using `os.path.join(base_path, filename)`.
+        """
+        filename = self.filename(class_name)
+        filepath = os.path.join(self.base_path, filename)
+        return filepath
 
     def classes(self):
         """
@@ -225,6 +212,13 @@ class Cobertura(object):
         # FIXME: this will lookup a list which is slow, make it O(1)
         return class_name in self.classes()
 
+    def class_lines(self, class_name):
+        """
+        Return a list for source lines of class `class_name`.
+        """
+        with open(self.filepath(class_name)) as f:
+            return f.readlines()
+
     def packages(self):
         """
         Return the list of available packages in the coverage report.
@@ -241,48 +235,116 @@ class CoberturaDiff(object):
         self.cobertura2 = cobertura2
 
     def diff_total_statements(self, class_name=None):
-        statements1 = self.cobertura1.total_statements(class_name)
+        if class_name is None:
+            statements1 = self.cobertura1.total_statements(class_name)
+            statements2 = self.cobertura2.total_statements(class_name)
+            return statements2 - statements1
+
+        if self.cobertura1.has_class(class_name):
+            statements1 = self.cobertura1.total_statements(class_name)
+        else:
+            statements1 = 0
+
         statements2 = self.cobertura2.total_statements(class_name)
+
         return statements2 - statements1
 
     def diff_total_misses(self, class_name=None):
-        misses1 = self.cobertura1.total_misses(class_name)
+        if class_name is None:
+            misses1 = self.cobertura1.total_misses(class_name)
+            misses2 = self.cobertura2.total_misses(class_name)
+            return misses2 - misses1
+
+        if self.cobertura1.has_class(class_name):
+            misses1 = self.cobertura1.total_misses(class_name)
+        else:
+            misses1 = 0
+
         misses2 = self.cobertura2.total_misses(class_name)
+
         return misses2 - misses1
 
     def diff_total_hits(self, class_name=None):
-        hits1 = self.cobertura1.total_hits(class_name)
+        if class_name is None:
+            hits1 = self.cobertura1.total_hits(class_name)
+            hits2 = self.cobertura2.total_hits(class_name)
+            return hits2 - hits1
+
+        if self.cobertura1.has_class(class_name):
+            hits1 = self.cobertura1.total_hits(class_name)
+        else:
+            hits1 = 0
+
         hits2 = self.cobertura2.total_hits(class_name)
+
         return hits2 - hits1
 
     def diff_line_rate(self, class_name=None):
-        rate1 = self.cobertura1.line_rate(class_name)
+        if class_name is None:
+            rate1 = self.cobertura1.line_rate(class_name)
+            rate2 = self.cobertura2.line_rate(class_name)
+            return rate2 - rate1
+
+        if self.cobertura1.has_class(class_name):
+            rate1 = self.cobertura1.line_rate(class_name)
+        else:
+            rate1 = 0.0
+
         rate2 = self.cobertura2.line_rate(class_name)
+
         return rate2 - rate1
 
+    def diff_missed_lines(self, class_name):
+        """
+        Return a list of 2-element tuples `(lineno, is_new)` where `lineno` is
+        a missed line number and `is_new` indicates whether the missed line was
+        introduced (True) or removed (False).
+        """
+        line_changed = []
+        for lineno, line, status in self.class_source(class_name):
+            if status is not None:
+                is_new = not status
+                line_changed.append((lineno, is_new))
+        return line_changed
+
+    def classes(self):
+        """
+        Return `self.cobertura2.classes()`.
+        """
+        return self.cobertura2.classes()
+
+    def filename(self, class_name):
+        """
+        Return `self.cobertura2.filename(class_name)`.
+        """
+        return self.cobertura2.filename(class_name)
+
     def class_source(self, class_name):
-        filename1 = self.cobertura1.filename(class_name)
-        filename2 = self.cobertura2.filename(class_name)
+        """
+        Return a list of 3-element tuples `(lineno, source, status)` for each
+        lines of code found in the source file of the class `class_name`.
 
-        for filename in (filename1, filename2):
-            if not os.path.exists(filename):
-                return [(0, '%s not found' % filename, None)]
+        The 3 elements in each tuple are:
+        `lineno`: line number in the source code
+        `source`: actual source code for line number `lineno`
+        `status`: True (hit), False (miss) or None (coverage unchanged)
+        """
+        if self.cobertura1.has_class(class_name):
+            lines1 = self.cobertura1.class_lines(class_name)
+            line_statuses1 = dict(self.cobertura1.line_statuses(class_name))
+        else:
+            lines1 = []
+            line_statuses1 = {}
 
-        try:
-            f1 = open(filename1)
-            f2 = open(filename2)
-            lines1 = f1.readlines()
-            lines2 = f2.readlines()
-        finally:
-            f1.close()
-            f2.close()
+#            if not os.path.exists(filename):
+#                return [(0, '%s not found' % filename, None)]
+
+        lines2 = self.cobertura2.class_lines(class_name)
+        line_statuses2 = dict(self.cobertura2.line_statuses(class_name))
 
         # Build a dict of lineno2 -> lineno1
         lineno_map = reconcile_lines(lines2, lines1)
 
-        line_statuses1 = dict(self.cobertura1.line_statuses(class_name))
-        line_statuses2 = dict(self.cobertura2.line_statuses(class_name))
-
         lines = []
         for lineno, line in enumerate(lines2, start=1):
 
@@ -304,3 +366,13 @@ class CoberturaDiff(object):
             lines.append((lineno, line, line_status))
 
         return lines
+
+    def class_source_hunks(self, class_name):
+        """
+        Like `CoberturaDiff.class_source`, but returns a list of line hunks of
+        the lines that have changed. An empty list means that the class has no
+        lines that have a change in coverage status.
+        """
+        lines = self.class_source(class_name)
+        hunks = hunkify_lines(lines)
+        return hunks
